local JSON = require "json"

-- Access the Lightroom SDK namespaces.
local LrDialogs = import 'LrDialogs'
local LrRecursionGuard = import 'LrRecursionGuard'
local LrLogger = import 'LrLogger'
local LrApplication = import 'LrApplication'
local LrApplicationView = import 'LrApplicationView'
local LrTasks = import 'LrTasks'
local LrFunctionContext = import 'LrFunctionContext'
local LrBinding = import 'LrBinding'
local LrView = import 'LrView'
local LrPrefs = import 'LrPrefs'
local LrErrors = import 'LrErrors'
local LrColor = import 'LrColor'
local LrHttp = import "LrHttp"
local LrMD5 = import "LrMD5"
local LrUndo = import 'LrUndo'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrSystemInfo = import 'LrSystemInfo'
local catalog = LrApplication.activeCatalog()
--local LrSystemInfo = import "LRSystemInfo"
local myLogger = LrLogger( 'exportLogger' )
local Info = require 'Info'
local LrDevelopController = import 'LrDevelopController'

myLogger:enable( "logfile" )
math.randomseed(os.time()) -- set the random seed to the current time

-- Write trace information to the logger.
debug = true

local function outputToLog( message )
	if (debug) then
		myLogger:trace( "ALEDEBUG: " .. (message or "NULL")  )
	end
end

local function dbgLog(message)
	if (debug) then
		outputToLog(message)
	end
end


function print_table(tbl, indent)
    if not indent then
        indent = 0
    end
    for k, v in pairs(tbl) do
        local formatting = string.rep("  ", indent) .. k .. ": "
        if type(v) == "table" then
            dbgLog(formatting)
            print_table(v, indent + 1)
        else
            dbgLog(formatting .. tostring(v))
        end
    end
end

-- print a table to the debug log
function printtbl( t,tab,lookup )
   
    local lookup = lookup or { [t] = 1 }
    local tab = tab or ""
    
    for i,v in pairs( t ) do
       outputToLog( tab..tostring(i), v )
       if type(i) == "table" and not lookup[i] then
          lookup[i] = 1
          outputToLog( tab.."Table: i" )
          printtbl( i,tab.."\t",lookup )
       end
       if type(v) == "table" and not lookup[v] then
          lookup[v] = 1
          outputToLog( tab.."Table: v" )
          printtbl( v,tab.."\t",lookup )
       end
    end
 end

 
--random 6 digit GUID generator.
 function generateRandomNumber()
    local number = ""
    for i = 1, 6 do
      local digit = math.random(1, 9) -- generate a digit between 1 and 9 (excluding 0)
      number = number .. tostring(digit) -- add the digit to the number string
    end
    return tonumber(number) -- convert the number string to a number and return it
  end
 
function loadPresetLibrary()
    dbgLog("Load Preset Library")
            
    currentPath = _PLUGIN.path
    dbgLog("Path:")
    currentPath = LrPathUtils.child(currentPath, "store.xml")
    dbgLog(currentPath)
    file = assert(io.open(currentPath, "r"))
    if (file == nil) then
        dbgLog("failed to open")
    
    else
        local jsonString = file:read("*all")
        presetStore = JSON:decode(jsonString)
        file:close()

        dbgLog("Preset Keys Loaded:")
        for key, _ in pairs(presetStore) do
            dbgLog(key)
        end
    end
end

 local function handleAdjustmentChange()
    -- Retrieve the currently selected photo
    dbgLog("Starting Handle Adjustment Change")
    local photo = LrApplication.activeCatalog():getTargetPhoto()
    -- Retrieve the Develop settings for the photo
    local settings = photo:getDevelopSettings()
    -- Retrieve the individual adjustments from the Develop settings and generate code
    
    optionCode = settings["HueAdjustmentRed"].. settings["HueAdjustmentOrange"] .. settings["HueAdjustmentYellow"] .. settings["HueAdjustmentGreen"] .. settings["HueAdjustmentAqua"]
    presetCode = settings["HueAdjustmentBlue"] .. settings["HueAdjustmentPurple"] .. settings["HueAdjustmentMagenta"]
    changeCodeNew =  optionCode.. presetCode
    dbgLog("ChangecodeOld :" .. changeCode)
    dbgLog("ChangecodeNew :" .. changeCodeNew)
    dbgLog("OptionCode :" .. optionCode)
    dbgLog("PresetCode :" .. presetCode)
    
    if (changeCodeNew ~= changeCode) then

        changeCode = changeCodeNew
        

        if (optionCode == "-9099-70900") then
            dbgLog("Load Preset - " .. presetCode)
            settings = presetStore[presetCode]
            
            if (settings == nil) then
                dbgLog("Preset Not Found in store. Preset = " .. presetCode)
                dbgLog("Presets in Store:")
                for key, _ in pairs(presetStore) do
                    dbgLog(key)
                end
                
            else
                dbgLog("Preset Found in store")
                local presetToApply = LrApplication.addDevelopPresetForPlugin( _PLUGIN, "Test", settings )
                --presetToApply = LrApplication.getDevelopPresetsForPlugin( _PLUGIN, "55BFDCDA-0AFA-4A1E-9991-E5039C6F52D0" )
                if presetToApply == nil then
                    dbgLog("Preset Not Generated correctly")
                else
                    dbgLog("Applying Preset")
                    if (LrUndo.canUndo()) then
                        LrUndo.undo()
                    end
                    catalog:withWriteAccessDo( "ApplyPreset", function()
                        --LrDevelopController.selectTool('loupe')
                        photo:applyDevelopPreset(presetToApply, _PLUGIN)
                            
                    end, {timeout = 15}
                    )
                end
            end
        

        elseif (optionCode == "-9599-70900") then
            loadPresetLibrary()
           
        
        elseif (optionCode == "-9699-70900") then
            dbgLog("Print Preset Library")

            print_table(presetStore)
           
        end


        if (changeCode == "9999999999999999") then
            dbgLog("Saving")
            retouch = settings["RetouchAreas"]
            --print_table(settings)
            newSettings = {}
            newSettings["RetouchAreas"] = settings["RetouchAreas"]
            

            presetGUID = generateRandomNumber()
            dbgLog(presetGUID)
            presetStore[tostring(presetGUID)]= newSettings

            local jsonSetting = JSON:encode(presetStore)
            dbgLog(jsonSetting)

            currentPath = _PLUGIN.path
            dbgLog("Path:")
            currentPath = LrPathUtils.child(currentPath, "store.xml")
            dbgLog(currentPath)
            file = assert(io.open(currentPath, "w"))
            if (file == nil) then
                dbgLog("failed to open")
            end

            dbgLog("Keys:")
            for key, _ in pairs(presetStore) do
                dbgLog(key)
            end

            local presetToApply = LrApplication.addDevelopPresetForPlugin( _PLUGIN, "Test", newSettings )
            --local jsonSetting = JSON:encode(presetToApply)
            file:write((jsonSetting))
            file:close()
            if (LrUndo.canUndo()) then
                LrUndo.undo()
            end

        end



        if (changeCode == "9999999999999997") then
            print_table(settings)
        end

            
        if (changeCode == "9999999999999998") then
            dbgLog("Recall")
            local photo = LrApplication.activeCatalog():getTargetPhoto()
            if (LrUndo.canUndo()) then
                LrUndo.undo()
            end

            settings = presetStore['181486']
            local presetToApply = LrApplication.addDevelopPresetForPlugin( _PLUGIN, "Test", settings )

            presetToApply = LrApplication.getDevelopPresetsForPlugin( _PLUGIN, "55BFDCDA-0AFA-4A1E-9991-E5039C6F52D0" )
            if presetToApply == nil then
                dbgLog("Preset Not Found")
                --local file = io.open("Blur.txt", "r")
                --local tableString = file:read("*all")
                --local newSettings = table.unserialize(tableString)
                --file:close()
            else

                catalog:withWriteAccessDo( "Test", function()
                    --LrDevelopController.selectTool('loupe')
                    photo:applyDevelopPreset(presetToApply, _PLUGIN)
                        
                end, {timeout = 30}
                )
            end
        end
        dbgLog('Adjustment change done')
    end

end

local function onAdjustmentChange(notification, context)
    
    LrTasks.startAsyncTask ( function ()
        local success, result = pcall(handleAdjustmentChange)
        if (success) then
            dbgLog("Handle Adjustent change successful")
        else
            dbgLog("Error Throwin in HandleAdjutsmentChange" .. tostring(result))
        end
  


end
)


end

local function startPlugin(context)
    dbgLog('Startup Function...')

    customPresets = {}
    LrFunctionContext.callWithContext ("Observer", function (context)
        if (pluginRunning == false) then
            dbgLog('Starting Monitor...')
            LrDevelopController.addAdjustmentChangeObserver (

                context, "onAdjustmentChange", onAdjustmentChange)

                pluginRunning = true
                while pluginRunning do 
                    LrTasks.sleep (1) 
                end
                dbgLog("Stopping Monitor")
        else
            dbgLog('Monitor Already Running')
        end
    end )
end



local function showWindow()
    local f = LrView.osFactory()


    local stopButton = f:push_button {
        title = "Stop",
        alignment = "center",
        action = function(submit_clicked)
            pluginRunning = false					
            
            end
    }
    local startButton = f:push_button {
        title = "Start",
        alignment = "center",
        action = function(submit_clicked)
                LrTasks.startAsyncTask(startPlugin)				 
            end
    }


    local c = f:column {
        spacing = f:dialog_spacing(),
        margin = 30,
        margin_top = 10,
        margin_bottom = 10,
        width  = 650,
        
        f:row {
            margin_left = Personalisation.LOGO_LEFT_MARGIN,
            f:picture {
                value = _PLUGIN:resourceId(Personalisation.LOGO_URL),
                width  = 500,
                height = 100,
                alignment = "right",
            },
        },
        f:row {
        },
        f:row {
            f:static_text {
                alignment = "center",
                fill_horizontal  = 1,
                title = "Please enter the email address you used to purchase your presets"
            },
        },
        f:row {
            height = 5,
        },

        f:row {
            margin_left = 280,
            startButton,
            stopButton
        },
        f:row {
            height = 20,
        },
        

        
        

    }

    retVal = LrDialogs.presentFloatingDialog (_PLUGIN, {
        title = windowTitle,
        contents = c,
        -- actionVerb = "Done", -- label for the action button
        --cancelVerb = "< exclude >" ,
        resizable = false,
        onShow = function(helpers)
            dbgLog("called open function")
                windowHelpers = helpers

        end,

        background_color = LrColor( 255, 255, 255),
    })
end


dbgLog("Starting Setting Monitor")
presetStore = {}
loadPresetLibrary()
changeCode = '0'
pluginRunning = false
dbgLog("Setup complete, starting UI")
showWindow()
LrTasks.startAsyncTask(startPlugin)